"""
This is the current Lino version number.

After editing this file, also edit the following files:

- Release notes : ../docs/releases/<__version__>.rst
- Releases overview: edit ../docs/releases/index.rst

Related public URLs:

- http://lino-framework.org/releases/
- http://pypi.python.org/pypi/lino

"""

__version__ = '1.5.11'
